#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){

	ofSetVerticalSync(true);

	ofBackground(255, 255, 255);
	
	playbackStartTime = 0;
	
	
	// 2 output channels,
	// 0 input channels
	// 22050 samples per second
	// 256 samples per buffer
	// 4 num buffers (latency)
	
	sampleRate 			= 44100;
	phase 				= 0;
	phaseAdder 			= 0.0f;
	phaseAdderTarget 	= 0.0f;
	volume				= 0.1f;
	bNoise 				= false;
	lAudio = new float[256];
	rAudio = new float[256];
	ofSoundStreamSetup(2,0,this, sampleRate,256, 4);
	
}


//--------------------------------------------------------------
void testApp::update(){

}

//--------------------------------------------------------------
void testApp::draw(){

	
	// -------------------------- draw the line
	
	for (int i=0; i<TPR_V.size(); i++) {
		TPR_V[i].draw();
	}
	
	for (int i=0; i<TPR_V.size(); i++) {
		// -------------------------- draw the point at the current time
		if (TPR_V[i].bHaveADrawing()){			// if we have a drawing to work with
			// figure out what time we are at, and make sure we playback cyclically (!)
			// use the duration here and make sure our timeToCheck is in the range of 0 - duration
			float timeToCheck = ofGetElapsedTimef() - playbackStartTime;
			while (timeToCheck > TPR_V[i].getDuration() && TPR_V[i].getDuration() > 0){
				timeToCheck -= TPR_V[i].getDuration();
			}
			
			// get the position and velocity at timeToCheck
			ofPoint pos = TPR_V[i].getPositionForTime(timeToCheck);
			
			
			float bgColor = ofMap(pos.y, 0, ofGetHeight(), 50, 200);
			ofBackground(bgColor, bgColor, bgColor);
			
			
			//sound from xy
			int distortion = ofMap( posX, 0, ofGetWidth(), 0, 44100);
			sampleRate = distortion;
			
			
			int width = ofGetWidth();
			pan = (float)pos.x / (float)pos.y;
			float height = (float)ofGetHeight();
			float heightPct = ((height-pos.y) / height);
			targetFrequency = 1000.0f * heightPct;
			phaseAdderTarget = (targetFrequency / (float) sampleRate) * TWO_PI;
			
			ofFill();
			ofSetColor(255,0,0);
			ofCircle(pos.x, pos.y, 10);
		}
		
	}
	
	
	ofSetColor(255,0,0);
	ofDrawBitmapString("height of dot controls background color", 30, 30);
	
	char reportString[255];
	sprintf(reportString, "volume: (%f) modify with -/+ keys\npan: (%f)\nsynthesis: %s", volume, pan, bNoise ? "noise" : "sine wave");
	if (!bNoise) sprintf(reportString, "%s (%fhz)", reportString, targetFrequency);
	ofDrawBitmapString(reportString,30,40);	
	
	char reportString2[255];
	
	int tpr_vLength = TPR_V.size();
	sprintf(reportString2, "Press 'c' to clear. Vector Size: (%i)", tpr_vLength);
	ofDrawBitmapString(reportString2,30,155);
	
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
	switch(key){
        case 'c':
            TPR_V.clear();
			break;
    }
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	posX = x;
	posY = y;
	volume	= 0.1f;
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

	TPR_V[TPR_V.size()-1].addPoint(x,y);
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	timePointRecorder temp;
	
	TPR_V.push_back(temp); 
	
	TPR_V[TPR_V.size()-1].startDrawing(x,y);
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	
	for (int i=0; i<TPR_V.size(); i++) {
		TPR_V[i].endDrawing();
	}
	
	playbackStartTime = ofGetElapsedTimef();
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){
}

//--------------------------------------------------------------
void testApp::audioRequested 	(float * output, int bufferSize, int nChannels){
	//pan = 0.5f;
	float leftScale = 1 - pan;
	float rightScale = pan;
	
	// sin (n) seems to have trouble when n is very large, so we
	// keep phase in the range of 0-TWO_PI like this:
	while (phase > TWO_PI){
		phase -= TWO_PI;
	}
	
	if ( bNoise == true){
		// ---------------------- noise --------------
		for (int i = 0; i < bufferSize; i++){
			lAudio[i] = output[i*nChannels    ] = ofRandomf() * volume * leftScale;
			rAudio[i] = output[i*nChannels + 1] = ofRandomf() * volume * rightScale;
		}
	} else {
		phaseAdder = 0.95f * phaseAdder + 0.05f * phaseAdderTarget;
		for (int i = 0; i < bufferSize; i++){
			phase += phaseAdder;
			float sample = sin(phase);
			lAudio[i] = output[i*nChannels    ] = sample * volume * leftScale;
			rAudio[i] = output[i*nChannels + 1] = sample * volume * rightScale;
		}
	}
	
}



