/*
 *  timePointRecorder.cpp
 *  drawingWithTime
 *
 *  Created by zachary lieberman on 9/21/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "timePointRecorder.h"


//--------------------------------------------------------------
timePointRecorder::timePointRecorder() {
	lAudio = new float[256];
	rAudio = new float[256];
	//ofAddListener(ofEvents.audioRequested, this, &timePointRecorder::audioRequested);
}


//-------------------------------------------------
void timePointRecorder::setup(){
	bRecording = false;
	
	// 2 output channels,
	// 0 input channels
	// 22050 samples per second
	// 256 samples per buffer
	// 4 num buffers (latency)
	
	sampleRate 			= 44100;
	phase 				= 0;
	phaseAdder 			= 0.0f;
	phaseAdderTarget 	= 0.0f;
	volume				= 0.5f;
	bNoise 				= false;
	lAudio = new float[256];
	rAudio = new float[256];
	ofSoundStreamSetup(2,0,this, sampleRate,256, 4);
	
	loopingSound.loadSound("drawbar_c4_a.aif");
	loopingSound.setVolume(10);
	loopingSound.setLoop(true);
	loopingSound.play();
}


//-------------------------------------------------
void timePointRecorder::update(){
	
}



//-------------------------------------------------
void timePointRecorder::clear(){
	pts.clear();
}

//-------------------------------------------------
void timePointRecorder::draw(){
	
	ofSetColor(0,0,0);
	ofNoFill();
	ofBeginShape();
	for (int i = 0; i < pts.size();i++){
		ofVertex(pts[i].x, pts[i].y);
	}
	ofEndShape();
	
	for (int i = 0; i < pts.size();i++){
		//sound from xy
		int distortion = ofMap( pts[i].x, 0, ofGetWidth(), 0, 44100);
		sampleRate = distortion;
		
		
		int width = ofGetWidth();
		pan = (float)pts[i].x / (float)pts[i].y;
		float height = (float)ofGetHeight();
		float heightPct = ((height-pts[i].y) / height);
		targetFrequency = 1000.0f * heightPct;
		phaseAdderTarget = (targetFrequency / (float) sampleRate) * TWO_PI;
		
	}

}

//-------------------------------------------------
void timePointRecorder::startDrawing(float x, float y){
	clear();
	bRecording = true;
	startTime = ofGetElapsedTimef();
	timePoint temp;
	temp.x = x;
	temp.y = y;
	temp.t = 0;
	pts.push_back(temp);
}

//-------------------------------------------------
void timePointRecorder::addPoint (float x, float y){
	timePoint temp;
	temp.x = x;
	temp.y = y;
	temp.t = ofGetElapsedTimef() - startTime;
	pts.push_back(temp);
}

//-------------------------------------------------
void timePointRecorder::endDrawing(){
	bRecording = false;
}

//-------------------------------------------------
bool timePointRecorder::bHaveADrawing(){
	if (bRecording == true){
		return false;
	} else if (pts.size() < 2){
		return false;
	}
	
	return true;
}


float timePointRecorder::getDuration(){
	float duration = 0;
	if (bHaveADrawing() == true){
		duration =  pts[pts.size()-1].t;
	}
	return duration;
}



//-------------------------------------------------
ofPoint	 timePointRecorder::getPositionForTime( float time){
	
	// are we recording?
	if (bHaveADrawing() == false){
		return ofPoint(0,0,0);
	}
	
	// now, let's figure out where we are in the drawing...
	ofPoint pos;
	for (int i = 0; i < pts.size()-1; i++){
		if (time >= pts[i].t && time <= pts[i+1].t){
			
			// calculate pct: 
			float part = time - pts[i].t;
			float whole = pts[i+1].t - pts[i].t;
			float pct = part / whole;
			
			// figure out where we are between a and b
			pos.x = (1-pct) * pts[i].x + (pct) * pts[i+1].x;
			pos.y = (1-pct) * pts[i].y + (pct) * pts[i+1].y;
	
			
		}
	}
	
	return pos;
	
}


//-------------------------------------------------
ofPoint	 timePointRecorder::getVelocityForTime( float time){
	
	// to get the velcoity, look back a bit of time, and at the current time
	// and get the difference 
	// veclocity = pos at time 1 - pos at time 0... 
	
	ofPoint prevPt = getPositionForTime( MAX(time - 0.09f, 0));		// check for where we were 0.05 seconds ago
	ofPoint currPt = getPositionForTime(time);							// check for where we are now.
	
	ofPoint diff;
	diff.x = currPt.x - prevPt.x;
	diff.y = currPt.y - prevPt.y;
	
	return diff;
}

//--------------------------------------------------------------
void timePointRecorder::audioRequested 	(float * output, int bufferSize, int nChannels){
	pan = 0.5f;
	float leftScale = 1 - pan;
	float rightScale = pan;
	
	// sin (n) seems to have trouble when n is very large, so we
	// keep phase in the range of 0-TWO_PI like this:
	while (phase > TWO_PI){
		phase -= TWO_PI;
	}
	
	if ( bNoise == true){
		// ---------------------- noise --------------
		for (int i = 0; i < bufferSize; i++){
			lAudio[i] = output[i*nChannels    ] = ofRandomf() * volume * leftScale;
			rAudio[i] = output[i*nChannels + 1] = ofRandomf() * volume * rightScale;
		}
	} else {
		phaseAdder = 0.95f * phaseAdder + 0.05f * phaseAdderTarget;
		for (int i = 0; i < bufferSize; i++){
			phase += phaseAdder;
			float sample = sin(phase);
			lAudio[i] = output[i*nChannels    ] = sample * volume * leftScale;
			rAudio[i] = output[i*nChannels + 1] = sample * volume * rightScale;
		}
	}
	
}
